--beginvalidatingquery 
IF EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = 'dbo' AND TABLE_NAME = 'SchemaVersion') 
    BEGIN 
    declare @major int = 7, @minor int = 4, @patch int = 1    
    IF EXISTS (SELECT 1 FROM dbo.SchemaVersion WHERE Major = @major AND Minor = @minor AND Patch = @patch) 
        select 0,'Already correct database version' 
    ELSE 
        select 1, 'Upgrading database' 
    END 
ELSE 
    select -1, 'Not an EPiServer Commerce database' 
--endvalidatingquery 
 
GO

-- recreate udttCatalogContentProperty 
EXEC sys.sp_rename 'dbo.udttCatalogContentProperty', 'temp_udttCatalogContentProperty';
GO
CREATE TYPE [dbo].[udttCatalogContentProperty] AS TABLE(
	[PropertyId] [bigint] NULL,
	[ObjectId] [int] NULL,
	[ObjectTypeId] INT NULL,
	[WorkId] [int] NULL,
	[MetaFieldId] [int] NOT NULL,
	[MetaClassId] int NOT NULL,
	[MetaFieldName] [nvarchar](255) NULL,
	[LanguageName] [nvarchar](50) NULL,
	[Boolean] [bit] NULL,
	[Number] [int] NULL,
	[FloatNumber] [decimal](38,9) NULL,
	[Money] [money] NULL,
	[Decimal] [decimal](38,9) NULL,
	[Date] [datetime] NULL,
	[Binary] [varbinary](max) NULL,
	[String] [nvarchar](450) NULL,
	[LongString] [nvarchar](max) NULL,
	[Guid] [uniqueidentifier] NULL,
	[IsNull] [bit] NULL,
	[IsEncrypted] [bit] NULL
	UNIQUE CLUSTERED ([ObjectId], [ObjectTypeId], [WorkId], [MetaFieldId], [LanguageName])
)
GO

DECLARE @Name NVARCHAR(1000);

DECLARE REF_CURSOR CURSOR FOR
SELECT referencing_schema_name + '.' + referencing_entity_name
FROM sys.dm_sql_referencing_entities('dbo.udttCatalogContentProperty', 'TYPE');

OPEN REF_CURSOR;

FETCH NEXT FROM REF_CURSOR INTO @Name;
WHILE (@@FETCH_STATUS = 0)
BEGIN
    EXEC sys.sp_refreshsqlmodule @name = @Name;
    FETCH NEXT FROM REF_CURSOR INTO @Name;
END;

CLOSE REF_CURSOR;
DEALLOCATE REF_CURSOR;
GO
DROP TYPE dbo.temp_udttCatalogContentProperty;
GO


IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id (N'[dbo].[CatalogContentProperty_SaveBatch]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1) DROP PROCEDURE [dbo].[CatalogContentProperty_SaveBatch] 
GO
 
CREATE PROCEDURE [dbo].[CatalogContentProperty_SaveBatch]
	@ContentProperty dbo.udttCatalogContentProperty readonly,
	@ContentExData dbo.udttCatalogContentEx readonly,
	@SyncVersion bit = 1
AS
BEGIN
	--delete items which are not in input
	DELETE A
	FROM [dbo].[CatalogContentProperty] A
	INNER JOIN @ContentProperty I ON	A.ObjectId = I.ObjectId AND 
									A.ObjectTypeId = I.ObjectTypeId AND
									A.LanguageName = I.LanguageName COLLATE DATABASE_DEFAULT AND
									A.MetaFieldId <> I.MetaFieldId

	--update encrypted field: support only LongString field
	DECLARE @propertyData dbo.udttCatalogContentProperty
	
	IF dbo.mdpfn_sys_IsAzureCompatible() = 0 -- Fields will be encrypted only when only when DB does not support Azure
		BEGIN
			EXEC mdpsp_sys_OpenSymmetricKey
		
			INSERT INTO @propertyData(ObjectId, ObjectTypeId, MetaFieldId, MetaClassId, MetaFieldName, LanguageName, Boolean, Number, 
						 FloatNumber, [Money], [Decimal], [Date], [Binary], [String], LongString, [Guid])
			SELECT ObjectId, ObjectTypeId, I.MetaFieldId, MetaClassId, I.MetaFieldName, I.[LanguageName], Boolean, Number, 
						 FloatNumber, [Money], [Decimal], [Date], [Binary], [String], 
						 CASE WHEN I.IsEncrypted = 1 THEN dbo.mdpfn_sys_EncryptDecryptString(I.LongString, 1) ELSE I.LongString END AS LongString, 
						 [Guid]
			FROM @ContentProperty I
			
			EXEC mdpsp_sys_CloseSymmetricKey
		END
	ELSE
		BEGIN
			INSERT INTO @propertyData(ObjectId, ObjectTypeId, MetaFieldId, MetaClassId, MetaFieldName, LanguageName, Boolean, Number, 
						 FloatNumber, [Money], [Decimal], [Date], [Binary], [String], LongString, [Guid])
			SELECT ObjectId, ObjectTypeId, I.MetaFieldId, MetaClassId, MetaFieldName, I.LanguageName, Boolean, Number, 
						 FloatNumber, [Money], [Decimal], [Date], [Binary], [String], LongString, [Guid]
			FROM @ContentProperty I
		END
    
	-- update/insert items which are not in input
	MERGE	[dbo].[CatalogContentProperty] as A
	USING	@propertyData as I
	ON		A.ObjectId = I.ObjectId AND 
			A.ObjectTypeId = I.ObjectTypeId AND 
			A.MetaFieldId = I.MetaFieldId AND
			A.LanguageName = I.LanguageName COLLATE DATABASE_DEFAULT
	WHEN	MATCHED 
		-- update the CatalogContentProperty for existing row
		THEN UPDATE SET
			A.Boolean = I.Boolean, 
			A.Number = I.Number, 
			A.FloatNumber = I.FloatNumber, 
			A.[Money] = I.[Money], 
			A.[Decimal] = I.[Decimal],
			A.[Date] = I.[Date], 
			A.[Binary] = I.[Binary], 
			A.[String] = I.[String], 
			A.LongString = I.LongString, 
			A.[Guid] = I.[Guid]

	WHEN NOT MATCHED BY TARGET 
		-- insert new record if the record does not exist in CatalogContentProperty table
		THEN 
			INSERT 
				(ObjectId, ObjectTypeId, MetaFieldId, MetaClassId, MetaFieldName, LanguageName, Boolean, Number, 
					FloatNumber, [Money], [Decimal], [Date], [Binary], [String], LongString, [Guid])
			VALUES
				(I.ObjectId, I.ObjectTypeId, I.MetaFieldId, I.MetaClassId, I.MetaFieldName, I.LanguageName, I.Boolean, I.Number, 
					I.FloatNumber, I.[Money], I.[Decimal], I.[Date], I.[Binary], I.[String], I.LongString, I.[Guid])
	;
	
END
GO

-- begin of creating mdpsp_sys_DeleteMetaKeyObjects sp
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id (N'[dbo].[mdpsp_sys_DeleteMetaKeyObjects]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1) DROP PROCEDURE [dbo].[mdpsp_sys_DeleteMetaKeyObjects] 
GO

CREATE PROCEDURE [dbo].[mdpsp_sys_DeleteMetaKeyObjects]
	@MetaClassId	INT,
	@MetaFieldId	INT	=	-1,
	@MetaObjectId	INT	=	-1,
	@WorkId			INT =	-1
AS
	
	CREATE TABLE #MetaKeysToRemove (MetaKey INT)
	INSERT INTO #MetaKeysToRemove (MetaKey)
		SELECT MK.MetaKey FROM MetaKey MK WHERE
		(@MetaObjectId = MK.MetaObjectId OR @MetaObjectId = -1)  AND
		(@MetaClassId = MK.MetaClassId OR @MetaClassId = -1) AND
		(@MetaFieldId = MK.MetaFieldId  OR @MetaFieldId = -1) AND
		(@WorkId = MK.WorkId OR @WorkId = -1)
	
	IF EXISTS (SELECT 1 FROM #MetaKeysToRemove)
	BEGIN
		-- Delete MetaObjectValue
		DELETE MO FROM MetaObjectValue MO INNER JOIN #MetaKeysToRemove M ON MO.MetaKey = M.MetaKey
		IF @@ERROR <> 0 GOTO ERR
		
		-- Delete MetaStringDictionaryValue
		DELETE MSD FROM MetaStringDictionaryValue MSD INNER JOIN #MetaKeysToRemove M ON MSD.MetaKey = M.MetaKey 
		IF @@ERROR <> 0 GOTO ERR
		
		--Delete MetaMultiValueDictionary
		DELETE MV FROM MetaMultiValueDictionary MV INNER JOIN #MetaKeysToRemove M ON MV.MetaKey = M.MetaKey
		IF @@ERROR <> 0 GOTO ERR
		
		--Delete MetaFileValue
		DELETE MF FROM MetaFileValue MF INNER JOIN #MetaKeysToRemove M ON MF.MetaKey = M.MetaKey
		IF @@ERROR <> 0 GOTO ERR
		
		--Delete MetaKey
		DELETE MK FROM MetaKey MK INNER JOIN #MetaKeysToRemove M ON MK.MetaKey = M.MetaKey
		IF @@ERROR <> 0 GOTO ERR
		
	END
	DROP TABLE #MetaKeysToRemove
ERR:
	IF object_id('tempdb..#MetaKeysToRemove') IS NOT NULL
	BEGIN
	DROP TABLE #MetaKeysToRemove
	END

	RETURN
GO
-- end of creating mdpsp_sys_DeleteMetaKeyObjects sp
 
--beginUpdatingDatabaseVersion 
 
INSERT INTO dbo.SchemaVersion(Major, Minor, Patch, InstallDate) VALUES(7, 4, 1, GETUTCDATE()) 
 
GO 

--endUpdatingDatabaseVersion 
